#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include"SimpleImage.h" /* 授業で使ってきた画像クラス */
#include"BMPIO.h" /* BMP画像の入出力関数 */
#include"DCT.h" /* 離散コサイン変換(DCT) */

/* 周波数画像を保存するための関数群 */
void Copy(Image *in,Image *out){
  int i,j;
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++)
      out->img[i][j]=log(1.0+fabs(in->img[i][j]*in->img[i][j]));
}

void Normalize(Image *out){
  double max,min;
  max=min=out->img[0][0];
  int i,j;
  for(i=0;i<out->sy;i++)
    for(j=0;j<out->sx;j++){
      if(max<out->img[i][j])max=out->img[i][j];
      if(min>out->img[i][j])min=out->img[i][j];
    }
  double range = fabs(max-min);if(range<=0.0)range=1.0;
    
  for(i=0;i<out->sy;i++)
    for(j=0;j<out->sx;j++){
      double val = out->img[i][j]-min;
      out->img[i][j] = 255.0*(val/range);
    }
}

void saveSpectrum(Image *R,Image *G,Image *B,char *filename){
  BMPIO *mybmp = new BMPIO();
  Image *dR = new Image(R->sx,R->sy);
  Image *dG = new Image(G->sx,G->sy);
  Image *dB = new Image(B->sx,B->sy);
  
  Copy(R,dR);
  Copy(G,dG);
  Copy(B,dB);
  Normalize(dR);
  Normalize(dG);
  Normalize(dB);
  
  mybmp->saveBMP(dR,dG,dB,filename);
  
  delete dR;
  delete dG;
  delete dB;
  delete mybmp;
}

//g++ -O3 testDCT.cxx -o testDCT
//./testDCT 入力BMP画像 出力画像名(ただし.bmpぬき) DCTブロックサイズ(int) カットする周波数のしきい値(int)


int main(int argc,char *argv[]){

  /* 変数の宣言 */
  BMPIO *mybmp = new BMPIO();
  Image *R= NULL;
  Image *G= NULL;
  Image *B= NULL;
  int sx,sy;
  
   /* argv[1]で与えられるファイル名の画像サイズの読み込み */
  if(mybmp->readBMPSize(&sx,&sy,argv[1])){
    /* メモリ確保 */
    R = new Image(sx,sy);
    G = new Image(sx,sy);
    B = new Image(sx,sy);
    
     /* argv[1]で与えられるファイル名の画像の読み込み */
    mybmp->readBMP(R,G,B,argv[1]);
    
    
   
  }else{
    delete mybmp;
    printf("Unsupported BMP format !\n");
    return 1;
  }
  

  /* DCTのブロックサイズを決める */
  int block_X_size,block_Y_size;
  int block_size = atoi(argv[3]);
  block_X_size = block_Y_size = block_size;


  /* DCT 順変換 */
  printf("Start DCT\n");
  Image *dctR = DCT(R,block_X_size,block_Y_size);
  Image *dctG = DCT(G,block_X_size,block_Y_size);
  Image *dctB = DCT(B,block_X_size,block_Y_size);
  printf("End DCT\n");
  char tmpout1[1000];
  sprintf(tmpout1,"%s_spectrum.bmp",argv[2]);
  saveSpectrum(dctR,dctG,dctB,tmpout1);
  
  int i,j,k,l;
  /* targetより上の周波数成分をカット(ゼロにする): ロウパスフィルタ */
  int target=atoi(argv[4]);
  
  /* ブロック(部分画像)毎のループ */
  for(i=0;i<dctR->sy;i+=block_Y_size)
    for(j=0;j<dctR->sx;j+=block_X_size){
      
      /* ブロック内部のループ: target以降の周波数を四角にカット(ゼロにする) */
      
      
      for(k=target;k<block_Y_size;k++)
	for(l=target;l<block_X_size;l++){
	  if((i+k)<dctR->sy&&(j+l)<dctR->sx){
	    dctR->img[(i+k)][(j+l)]=0.0;
	    dctG->img[(i+k)][(j+l)]=0.0;
	    dctB->img[(i+k)][(j+l)]=0.0;
	  }
	}
      for(k=0;k<block_Y_size;k++)
	for(l=target;l<block_X_size;l++){
	  if((i+k)<dctR->sy&&(j+l)<dctR->sx){
	    dctR->img[(i+k)][(j+l)]=0.0;
	    dctG->img[(i+k)][(j+l)]=0.0;
	    dctB->img[(i+k)][(j+l)]=0.0;
	  }
	}
      for(k=target;k<block_Y_size;k++)
	for(l=0;l<block_X_size;l++){
	  if((i+k)<dctR->sy&&(j+l)<dctR->sx){
	    dctR->img[(i+k)][(j+l)]=0.0;
	    dctG->img[(i+k)][(j+l)]=0.0;
	    dctB->img[(i+k)][(j+l)]=0.0;
	  }
	}
      

     
    }

  /* 周波数画像の保存 */
  char tmpout2[1000];
  sprintf(tmpout2,"%s_smooth_spectrum.bmp",argv[2]);
  saveSpectrum(dctR,dctG,dctB,tmpout2);
  printf("Start Inverse DCT\n");

  /* DCT 逆変換 */
  InverseDCT(dctR,R,block_X_size,block_Y_size);
  InverseDCT(dctG,G,block_X_size,block_Y_size);
  InverseDCT(dctB,B,block_X_size,block_Y_size);
  printf("End Inverse DCT\n");

  /* 逆変換が8bit:0-255になるとは限らないので*/
  for(i=0;i<R->sy;i++)
    for(j=0;j<R->sx;j++){
      if(R->img[i][j]<0.0)R->img[i][j]=0.0;
      if(R->img[i][j]>255.0)R->img[i][j]=255.0;
      if(G->img[i][j]<0.0)G->img[i][j]=0.0;
      if(G->img[i][j]>255.0)G->img[i][j]=255.0;
      if(B->img[i][j]<0.0)B->img[i][j]=0.0;
      if(B->img[i][j]>255.0)B->img[i][j]=255.0;
      
    }
  
  /* 画像に保存 */
  char tmpout3[1000];
  sprintf(tmpout3,"%s_smooth.bmp",argv[2]);
  mybmp->saveBMP(R,G,B,tmpout3);
  

  /* メモリ開放 */
  
  delete dctR;
  delete dctG;
  delete dctB;
  delete mybmp;
  if(R!=NULL)delete R;
  if(G!=NULL)delete G;
  if(B!=NULL)delete B;
 
  return 0;
}

